import iSATLibrary.ISatSolver;
import java.util.LinkedHashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Langford {

	 public ArrayList<String> symbol = new ArrayList<String>();

	 public ArrayList<Integer> getInclude(String str){
		  ArrayList<Integer> litList = new ArrayList<Integer>();
		  for(int i = 0; i<symbol.size(); i++){
				if(symbol.get(i).contains(str+"_")) litList.add(i+1);
		  }
		  return litList;
	 }

	 public void encode(ISatSolver solver, int num){
		  ArrayList<String> strList = new ArrayList<String>();
		  for(int i=1; i<= num; i++) strList.add("d"+i);
		  for(int i=1; i<= 2*num; i++) strList.add("s"+i);
		  int even = 1-num%2;
		  int k;
		  for(int i=1; i<=num; i++){
				for(int j=1; j<=2*num-1; j++){
					 k = i+j+1;
					 if((i != num-even || j <= num/2) && k<=2*num){
						  symbol.add("d"+i+"_s"+j+"_s"+k+"_");
					 }
				}
		  }
		  for(String str: strList){
				ArrayList<Integer> litList = getInclude(str);
				int size = litList.size();
				int[] atmost = new int[2];
				int[] atleast = new int[size];
				for(int i=0; i<size; i++){
					 atleast[i] = litList.get(i);
				}
				solver.add_clause(atleast);
				for(int i=0; i<size; i++){
					 for(int j=i+1; j<size; j++){
						  atmost[0] = -litList.get(i);
						  atmost[1] = -litList.get(j);
						  solver.add_clause(atmost);
					 }
				}
		  }
	 }
	 
	 public boolean solve(ISatSolver solver){
		  return solver.solve();
	 }

	 public void decode(ISatSolver solver, int num){
		  System.out.println("SATISFIABLE");
		  int[] model = solver.get_int_model();
 		  int[] seq = new int[2*num];
		  Pattern p = Pattern.compile("d(\\d+)_s(\\d+)_s(\\d+)_");
		  Matcher m;
		  for(int i : model){
				if(i > 0){
					 String str = symbol.get(i-1);
					 m = p.matcher(str);
					 if(m.find()){
						  int n = Integer.parseInt(m.group(1));
						  int k = Integer.parseInt(m.group(2));
						  int l = Integer.parseInt(m.group(3));
						  seq[k-1] = seq[l-1] = n;
					 }
				}
		  }
		  for(int i : seq) System.out.print(i+" ");
		  System.out.println();
	 }

	 public static void main(String[] args) {
		Langford lp = new Langford();
		ISatSolver solver = new ISatSolver("glueminisat");
		if(args.length != 1){
			 System.out.println("Usage : Langford (number)");
			 System.exit(1);
		}
		int num = Integer.parseInt(args[0]);
		solver.set_verbosity(0);
		lp.encode(solver, num);
		if (lp.solve(solver)) lp.decode(solver, num); else System.out.println("UNSATISFIABLE");
	 }

}
