
#include "ipasir.h"
#include "Solver.h"
//#include "simp/SimpSolver.h"
#include "ExtendedAPI.h"

using namespace std;
using namespace Minisat;

extern "C" {
  static const char * sig = "glueminisat2.2.5";
#include <sys/resource.h>
#include <sys/time.h>
  static double getime(void) {
	 struct rusage u;
	 double res;
	 //	 if (getrusage (RUSAGE_SELF, &u)) return 0;
	 getrusage(RUSAGE_SELF, &u);
	 res = u.ru_utime.tv_sec + 1e-6 * u.ru_utime.tv_usec;
	 res += u.ru_stime.tv_sec + 1e-6 * u.ru_stime.tv_usec;
	 return res;
  }
};

class IPAsirGlueMiniSat : public Solver {
  vec<Lit> assumptions, clause;
  int szfmap; unsigned char * fmap; bool nomodel;
  unsigned long long calls;
  //double solvetime;
  void reset () { if (fmap) delete [] fmap, fmap = 0, szfmap = 0; }
  Lit import(int lit){
	 while (abs(lit) > nVars()) (void) newVar();
	 return mkLit(Var (abs(lit)-1), (lit > 0));
  }
  void ana(){
	 fmap = new unsigned char [szfmap = nVars()];
	 memset (fmap, 0, szfmap);
	 for (int i=0; i<conflict.size(); i++){
		int tmp = var (conflict[i]);
		assert(0<=tmp && tmp<szfmap);
		fmap[tmp] = 1;
	 }
  }
  double ps (double s, double t) {return t ? s/t : 0; }
public:
  IPAsirGlueMiniSat () : szfmap (0), fmap (0), nomodel (false), calls (0) {
    // MiniSAT by default produces non standard conforming messages.
    // So either we have to set this to '0' or patch the sources.
    verbosity = 1;
  }
  ~IPAsirGlueMiniSat () { reset (); }
  void add (int lit) {
    reset ();
    nomodel = true;
    if (lit) clause.push (import (lit));
    else addClause (clause), clause.clear ();
  }
  void assume (int lit) {
    reset ();
    nomodel = true;
    assumptions.push (import (lit));
  }
  int solve () {
    calls++;
    reset ();
	 //solvetime = getime();
    lbool res = solveLimited (assumptions);
    assumptions.clear ();
    nomodel = (res != l_True);
    return (res == l_Undef) ? 0 : (res == l_True ? 10 : 20);
  }
  int val (int lit) {
    if (nomodel) return 0;
    lbool res = modelValue (import (lit));
    return (res == l_True) ? lit : -lit;
  }
  int failed (int lit) {
    if (!fmap) ana ();
    int tmp = var (import (lit));
    assert (0 <= tmp && tmp < nVars ());
    return fmap[tmp] != 0;
  }
  void stats () {
    double t = getime ();
    printf (
      "c [%s]\n"
      "c [%s]        calls %12llu   %9.1f per second\n"
      "c [%s]     restarts %12llu   %9.1f per second\n"
      "c [%s]    conflicts %12llu   %9.1f per second\n"
      "c [%s]    decisions %12llu   %9.1f per second\n"
      "c [%s] propagations %12llu   %9.1f per second\n"
      "c [%s]\n",
    sig,
    sig, (unsigned long long) calls, ps (calls, t),
    sig, (unsigned long long) starts, ps (starts, t),
    sig, (unsigned long long) conflicts, ps (conflicts, t),
    sig, (unsigned long long) decisions, ps (decisions, t),
    sig, (unsigned long long) propagations, ps (propagations, t),
    sig);
    fflush (stdout);
  } 
  double time () {
	 return getime();
	 //return solvetime;
  }
  /*double totaltime () {
	 return getime();
	 }*/
};

// added for iSugar

class IPAsirGlueMiniSatExtended : public IPAsirGlueMiniSat {
  typedef IPAsirGlueMiniSat base;
public:
  void add_clauses(int * clauses, int length){
	 for (int i=0; i<length; i++) {
		base::add(clauses[i]);
	 }
  }
  int * model(){
	 int vars = nVars();
	 int * ansModel = (int *)malloc(sizeof(int)*vars);
	 for(int i=0; i<vars; i++){
		ansModel[i] = val(i+1);
	 }
	 return ansModel;
  }
  void add_blocking_clause(int * literals, int length){
	 // all variable
	 if (literals == NULL){
		int * ansModel = model();
		for(int i=0; i<nVars(); i++)
		  base::add(-ansModel[i]);
		base::add(0);
	 }
	 else{
		for(int i=0; i<length; i++){
		  base::add(-literals[i]);
		}
		base::add(0);
	 }
  }
  void release(int * p){
	 free(p);
  }
  double gettime(){
	 return base::time(); 
  }
  /*double gettotaltime(){
	 return base::totaltime(); 
	 }*/
  void setFrozen(int literal){
	 // for only "simp" version
	 //base::setFrozen(literal, true);
  }
  int isEliminated(int literal){
	 // for only "simp" version
	 // return base::isEliminated(literal) ? 1 : 0;
	 return 0;
  }
};

static IPAsirGlueMiniSatExtended * import (void * s) { return (IPAsirGlueMiniSatExtended*) s; }
// added for iSugar
void isat_set_verbosity(void * s, int v){ import(s) -> verbosity = v; }
int isat_vars(void * s){ return import(s) -> nVars(); }
void isat_add_clauses(void * s, int * clauses, int length){ import(s) -> add_clauses(clauses,length);}
int * isat_model(void * s){ return import(s) -> model();}
void isat_add_blocking_clause(void * s, int * literals, int length){ import(s) -> add_blocking_clause(literals,length);}
void isat_clean(void * s, int * p){ import(s) -> release(p);}
double isat_get_time(void * s){ return import(s) -> gettime();}
//double isat_get_total_time(void * s){ return import(s) -> gettotaltime();}
// void isat_add_PB(void * s, int * coefficients, int * variables);
// int * isat_MUS(void * s);
void isat_freeze(void * s, int literal){ import(s) -> setFrozen(literal); }
int isat_is_eliminated(void * s, int literal){ return import(s) -> isEliminated(literal);}
// void ipasir_delete_learnts(void * s);
// void ipasir_set_polarity(void * s, int * polarity);
// int * ipasir_polarity(void * s);
